@@ -8,7 +8,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: chmod.c,v 1.11 2004/04/18 06:05:34 ralf Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -34,7 +34,7 @@ int chmod(
   rtems_filesystem_location_info_t loc;
   int                              result;
 
-  status = rtems_filesystem_evaluate_path( path, 0, &loc, TRUE );
+  status = rtems_filesystem_evaluate_path( path, strlen( path ), 0, &loc, true );
   if ( status != 0 )
     return -1;
 
