@@ -1,20 +1,20 @@
-/* $Id: libi2c.c,v 1.3.2.1 2007/10/11 12:46:58 joel Exp $ */
+/* $Id$ */
 
 /* libi2c Implementation */
 
-/* 
+/*
  * Authorship
  * ----------
  * This software was created by
  *     Till Straumann <strauman@slac.stanford.edu>, 2005,
  * 	   Stanford Linear Accelerator Center, Stanford University.
- * 
+ *
  * Acknowledgement of sponsorship
  * ------------------------------
  * This software was produced by
  *     the Stanford Linear Accelerator Center, Stanford University,
  * 	   under Contract DE-AC03-76SFO0515 with the Department of Energy.
- * 
+ *
  * Government disclaimer of liability
  * ----------------------------------
  * Neither the United States nor the United States Department of Energy,
@@ -23,18 +23,18 @@
  * completeness, or usefulness of any data, apparatus, product, or process
  * disclosed, or represents that its use would not infringe privately owned
  * rights.
- * 
+ *
  * Stanford disclaimer of liability
  * --------------------------------
  * Stanford University makes no representations or warranties, express or
  * implied, nor assumes any liability for the use of this software.
- * 
+ *
  * Stanford disclaimer of copyright
  * --------------------------------
  * Stanford University, owner of the copyright, hereby disclaims its
  * copyright and all other rights in this software.  Hence, anyone may
- * freely use it for any purpose without restriction.  
- * 
+ * freely use it for any purpose without restriction.
+ *
  * Maintenance of notices
  * ----------------------
  * In the interest of clarity regarding the origin and status of this
@@ -43,9 +43,13 @@
  * or distributed by the recipient and are to be affixed to any copy of
  * software made or distributed by the recipient that contains a copy or
  * derivative of this software.
- * 
+ *
  * ------------------ SLAC Software Notices, Set 4 OTT.002a, 2004 FEB 03
- */ 
+ */
+/*
+ * adaptations to also handle SPI devices
+ * by Thomas Doerfler, embedded brains GmbH, Puchheim, Germany
+ */
 #if HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -55,14 +59,16 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <assert.h>
+#include <stdarg.h>
 
 #include <rtems.h>
 #include <rtems/error.h>
+#include <rtems/bspIo.h>
 #include <rtems/libio.h>
 
 #include <rtems/libi2c.h>
 
-#define DRVNM "libi2c:"
+#define DRVNM "libi2c: "
 
 #define MAX_NO_BUSSES	8       /* Also limited by the macro building minor numbers */
 #define MAX_NO_DRIVERS	16      /* Number of high level drivers we support          */
@@ -91,28 +97,29 @@
 
 #define DISPATCH(rval, entry, dflt)	\
 	do {				\
-		rtems_driver_address_table *ops = drvs[--drv].drv->ops;	\
+		const rtems_driver_address_table *ops = drvs[--drv].drv->ops;	\
 		rval = ops->entry ? ops->entry(major,minor,arg) : dflt;	\
 	} while (0)
 
 
 rtems_device_major_number rtems_libi2c_major;
 
+static bool is_initialized = false;
+
 static struct i2cbus
 {
   rtems_libi2c_bus_t *bush;
   volatile rtems_id mutex;      /* lock this across start -> stop */
-  volatile short waiting;
-  volatile char started;
+  volatile bool started;
   char *name;
-} busses[MAX_NO_BUSSES] = { { 0 } };
+} busses[MAX_NO_BUSSES] = { { NULL, RTEMS_ID_NONE, false, NULL } };
 
 static struct
 {
   rtems_libi2c_drv_t *drv;
-} drvs[MAX_NO_DRIVERS] = { { 0} };
+} drvs[MAX_NO_DRIVERS] = { { NULL } };
 
-static rtems_id libmutex = 0;
+static rtems_id libmutex = RTEMS_ID_NONE;
 
 #define LOCK(m)		assert(!rtems_semaphore_obtain((m), RTEMS_WAIT, RTEMS_NO_TIMEOUT))
 #define UNLOCK(m)	rtems_semaphore_release((m))
@@ -127,17 +134,34 @@ static rtems_id libmutex = 0;
 	 |RTEMS_NO_PRIORITY_CEILING	\
 	 |RTEMS_LOCAL )
 
-static rtems_id
-mutexCreate (rtems_name nm)
+/* During early stages of life, stdio is not available */
+
+static void
+safe_printf (const char *fmt, ...)
+{
+va_list ap;
+
+	va_start(ap, fmt);
+	if ( _System_state_Is_up( _System_state_Get() ) )
+		vfprintf( stderr, fmt, ap );
+	else
+		vprintk( fmt, ap );
+	va_end(ap);
+}
+
+static rtems_status_code
+mutexCreate (rtems_name nm, rtems_id *pm)
 {
   rtems_status_code sc;
-  rtems_id rval;
+
   if (RTEMS_SUCCESSFUL !=
-      (sc = rtems_semaphore_create (nm, 1, MUTEX_ATTS, 0, &rval))) {
-    rtems_error (sc, DRVNM " unable to create mutex\n");
-    return 0;
+      (sc = rtems_semaphore_create (nm, 1, MUTEX_ATTS, 0, pm))) {
+	if ( _System_state_Is_up( _System_state_Get() ) )
+    	rtems_error (sc, DRVNM "Unable to create mutex\n");
+	else
+		printk (DRVNM "Unable to create mutex (status code %i)\n", sc);
   }
-  return rval;
+  return sc;
 }
 
 /* Lock a bus avoiding to have a mutex, which is mostly
@@ -151,19 +175,28 @@ static void
 lock_bus (int busno)
 {
   struct i2cbus *bus = &busses[busno];
-  LIBLOCK ();
-  if (!bus->waiting) {
-    /* nobody is holding the bus mutex - it's not there. Create it on the fly */
-    if (!
-        (bus->mutex =
-         mutexCreate (rtems_build_name ('i', '2', 'c', '0' + busno)))) {
-      LIBUNLOCK ();
-      rtems_panic (DRVNM " unable to create bus lock");
+
+  if (bus->mutex == RTEMS_ID_NONE) {
+    /*
+     * Nobody is holding the bus mutex - it's not there.  Create it on the fly.
+     */
+    LIBLOCK ();
+    if (bus->mutex == RTEMS_ID_NONE) {
+      rtems_id m = RTEMS_ID_NONE;
+      rtems_status_code sc = mutexCreate (
+        rtems_build_name ('i', '2', 'c', '0' + busno),
+        &m
+      );
+      if (sc != RTEMS_SUCCESSFUL) {
+        LIBUNLOCK ();
+        rtems_panic (DRVNM "Unable to create bus lock");
+        return;
+      }
+      bus->mutex = m;
     }
+    LIBUNLOCK ();
   }
-  /* count number of people waiting on this bus; only the last one deletes the mutex */
-  bus->waiting++;
-  LIBUNLOCK ();
+
   /* Now lock this bus */
   LOCK (bus->mutex);
 }
@@ -172,33 +205,33 @@ static void
 unlock_bus (int busno)
 {
   struct i2cbus *bus = &busses[busno];
-  LIBLOCK ();
   UNLOCK (bus->mutex);
-  if (!--bus->waiting) {
-    rtems_semaphore_delete (bus->mutex);
-  }
-  LIBUNLOCK ();
 }
 
 /* Note that 'arg' is always passed in as NULL */
-static rtems_status_code
-i2c_init (rtems_device_major_number major, rtems_device_minor_number minor,
+rtems_status_code
+rtems_i2c_init (rtems_device_major_number major, rtems_device_minor_number minor,
           void *arg)
 {
   rtems_status_code rval;
+  /* No busses or drivers can be registered at this point;
+   * avoid the macro aborting with an error
   DECL_CHECKED_DRV (drv, busno, minor)
+   */
 
-    if (0 == drv) {
-    rval = 0;
+  rval = mutexCreate (rtems_build_name ('l', 'I', '2', 'C'), &libmutex);
+
+  if ( RTEMS_SUCCESSFUL == rval ) {
+  	is_initialized     = true;
+	rtems_libi2c_major = major;
   } else {
-    /* this is probably never called */
-    DISPATCH (rval, initialization_entry, RTEMS_SUCCESSFUL);
+  	libmutex = RTEMS_ID_NONE;
   }
   return rval;
 }
 
-static rtems_status_code
-i2c_open (rtems_device_major_number major, rtems_device_minor_number minor,
+rtems_status_code
+rtems_i2c_open (rtems_device_major_number major, rtems_device_minor_number minor,
           void *arg)
 {
   rtems_status_code rval;
@@ -212,8 +245,8 @@ i2c_open (rtems_device_major_number major, rtems_device_minor_number minor,
   return rval;
 }
 
-static rtems_status_code
-i2c_close (rtems_device_major_number major, rtems_device_minor_number minor,
+rtems_status_code
+rtems_i2c_close (rtems_device_major_number major, rtems_device_minor_number minor,
            void *arg)
 {
   rtems_status_code rval;
@@ -227,8 +260,8 @@ i2c_close (rtems_device_major_number major, rtems_device_minor_number minor,
   return rval;
 }
 
-static rtems_status_code
-i2c_read (rtems_device_major_number major, rtems_device_minor_number minor,
+rtems_status_code
+rtems_i2c_read (rtems_device_major_number major, rtems_device_minor_number minor,
           void *arg)
 {
   int rval;                     /* int so we can check for negative value */
@@ -257,8 +290,8 @@ i2c_read (rtems_device_major_number major, rtems_device_minor_number minor,
   return rval;
 }
 
-static rtems_status_code
-i2c_write (rtems_device_major_number major, rtems_device_minor_number minor,
+rtems_status_code
+rtems_i2c_write (rtems_device_major_number major, rtems_device_minor_number minor,
            void *arg)
 {
   int rval;                     /* int so we can check for negative value */
@@ -287,8 +320,8 @@ i2c_write (rtems_device_major_number major, rtems_device_minor_number minor,
   return rval;
 }
 
-static rtems_status_code
-i2c_ioctl (rtems_device_major_number major, rtems_device_minor_number minor,
+rtems_status_code
+rtems_i2c_ioctl (rtems_device_major_number major, rtems_device_minor_number minor,
            void *arg)
 {
   rtems_status_code rval;
@@ -304,31 +337,43 @@ i2c_ioctl (rtems_device_major_number major, rtems_device_minor_number minor,
 
 
 /* Our ops just dispatch to the registered drivers */
-static rtems_driver_address_table libi2c_io_ops = {
-  initialization_entry:  i2c_init,
-  open_entry:            i2c_open,
-  close_entry:           i2c_close,
-  read_entry:            i2c_read,
-  write_entry:           i2c_write,
-  control_entry:         i2c_ioctl,
+const rtems_driver_address_table rtems_libi2c_io_ops = {
+  initialization_entry:  rtems_i2c_init,
+  open_entry:            rtems_i2c_open,
+  close_entry:           rtems_i2c_close,
+  read_entry:            rtems_i2c_read,
+  write_entry:           rtems_i2c_write,
+  control_entry:         rtems_i2c_ioctl,
 };
 
-
 int
-rtems_libi2c_initialize ()
+rtems_libi2c_initialize (void)
 {
   rtems_status_code sc;
 
-  if (!(libmutex = mutexCreate (rtems_build_name ('l', 'I', '2', 'C'))))
-    return -1;
+  if (is_initialized) {
+    /*
+     * already called before? then skip this step
+     */
+    return 0;
+  }
 
-  sc = rtems_io_register_driver (0, &libi2c_io_ops, &rtems_libi2c_major);
+  /* rtems_io_register_driver does NOT currently check nor report back
+   * the return code of the 'init' operation, so we cannot
+   * rely on return code since it may seem OK even if the driver 'init;
+   * op failed.
+   * Let 'init' handle 'is_initialized'...
+   */
+  sc = rtems_io_register_driver (0, &rtems_libi2c_io_ops, &rtems_libi2c_major);
   if (RTEMS_SUCCESSFUL != sc) {
-    fprintf (stderr,
-             DRVNM " Claiming driver slot failed (rtems status code %i)\n",
+    safe_printf(
+             DRVNM "Claiming driver slot failed (rtems status code %i)\n",
              sc);
-    rtems_semaphore_delete (libmutex);
-    libmutex = 0;
+    if (libmutex != RTEMS_ID_NONE) {
+      rtems_semaphore_delete (libmutex);
+    }
+    libmutex = RTEMS_ID_NONE;
+    is_initialized = false;
     return -1;
   }
 
@@ -336,7 +381,7 @@ rtems_libi2c_initialize ()
 }
 
 int
-rtems_libi2c_register_bus (char *name, rtems_libi2c_bus_t * bus)
+rtems_libi2c_register_bus (const char *name, rtems_libi2c_bus_t * bus)
 {
   int i;
   rtems_status_code err;
@@ -349,13 +394,14 @@ rtems_libi2c_register_bus (char *name, rtems_libi2c_bus_t * bus)
 
   /* check */
   if ('/' != *nmcpy) {
-    fprintf (stderr,
-             "Bad name; must be an absolute path starting with '/'\n");
+    safe_printf ( DRVNM "Bad name: must be an absolute path starting with '/'\n");
+    free( nmcpy );
     return -RTEMS_INVALID_NAME;
   }
   /* file must not exist */
   if (!stat (nmcpy, &sbuf)) {
-    fprintf (stderr, "Bad name; file exists already\n");
+    safe_printf ( DRVNM "Bad name: file exists already\n");
+    free( nmcpy );
     return -RTEMS_INVALID_NAME;
   }
 
@@ -366,20 +412,23 @@ rtems_libi2c_register_bus (char *name, rtems_libi2c_bus_t * bus)
   i = stat (nmcpy, &sbuf);
   *chpt = tmp;
   if (i) {
-    fprintf (stderr, "Get %s status failed: %s\n",
-             nmcpy,strerror(errno));
+    safe_printf ( DRVNM "Get %s status failed: %s\n",
+             nmcpy, strerror(errno));
+    free( nmcpy );
     return -RTEMS_INVALID_NAME;
   }
   /* should be a directory since name terminates in '/' */
 
 
-  if (!libmutex) {
-    fprintf (stderr, DRVNM " library not initialized\n");
+  if (libmutex == RTEMS_ID_NONE) {
+    safe_printf ( DRVNM "Library not initialized\n");
+    free( nmcpy );
     return -RTEMS_NOT_DEFINED;
   }
 
-  if (bus->size < sizeof (*bus)) {
-    fprintf (stderr, DRVNM " bus-ops size too small -- misconfiguration?\n");
+  if (bus == NULL || bus->size < sizeof (*bus)) {
+    safe_printf ( DRVNM "No bus-ops or size too small -- misconfiguration?\n");
+    free( nmcpy );
     return -RTEMS_NOT_CONFIGURED;
   }
 
@@ -388,9 +437,8 @@ rtems_libi2c_register_bus (char *name, rtems_libi2c_bus_t * bus)
     if (!busses[i].bush) {
       /* found a free slot */
       busses[i].bush = bus;
-      busses[i].mutex = 0;
-      busses[i].waiting = 0;
-      busses[i].started = 0;
+      busses[i].mutex = RTEMS_ID_NONE;
+      busses[i].started = false;
 
       if (!name)
         sprintf (nmcpy + strlen (nmcpy), "%i", i);
@@ -428,7 +476,7 @@ not_started (int busno)
 }
 
 rtems_status_code
-rtems_libi2c_send_start (uint32_t minor)
+rtems_libi2c_send_start (rtems_device_minor_number minor)
 {
   int rval;
   DECL_CHECKED_BH (busno, bush, minor, +)
@@ -453,13 +501,13 @@ rtems_libi2c_send_start (uint32_t minor)
     unlock_bus (busno);
   } else {
     /* successful 1st start; keep bus locked until stop is sent */
-    busses[busno].started = 1;
+    busses[busno].started = true;
   }
   return rval;
 }
 
 rtems_status_code
-rtems_libi2c_send_stop (uint32_t minor)
+rtems_libi2c_send_stop (rtems_device_minor_number minor)
 {
   rtems_status_code rval;
   DECL_CHECKED_BH (busno, bush, minor, +)
@@ -469,14 +517,14 @@ rtems_libi2c_send_stop (uint32_t minor)
 
   rval = bush->ops->send_stop (bush);
 
-  busses[busno].started = 0;
+  busses[busno].started = false;
 
   unlock_bus (busno);
   return rval;
 }
 
 rtems_status_code
-rtems_libi2c_send_addr (uint32_t minor, int rw)
+rtems_libi2c_send_addr (rtems_device_minor_number minor, int rw)
 {
   rtems_status_code sc;
   DECL_CHECKED_BH (busno, bush, minor, +)
@@ -491,12 +539,14 @@ rtems_libi2c_send_addr (uint32_t minor, int rw)
 }
 
 int
-rtems_libi2c_read_bytes (uint32_t minor, unsigned char *bytes, int nbytes)
+rtems_libi2c_read_bytes (rtems_device_minor_number minor,
+			 unsigned char *bytes,
+			 int nbytes)
 {
   int sc;
   DECL_CHECKED_BH (busno, bush, minor, -)
 
-    if (not_started (busno))
+  if (not_started (busno))
     return -RTEMS_NOT_OWNER_OF_RESOURCE;
 
   sc = bush->ops->read_bytes (bush, bytes, nbytes);
@@ -506,25 +556,100 @@ rtems_libi2c_read_bytes (uint32_t minor, unsigned char *bytes, int nbytes)
 }
 
 int
-rtems_libi2c_write_bytes (uint32_t minor, unsigned char *bytes, int nbytes)
+rtems_libi2c_write_bytes (rtems_device_minor_number minor,
+			  const unsigned char *bytes,
+			  int nbytes)
 {
   int sc;
   DECL_CHECKED_BH (busno, bush, minor, -)
 
-    if (not_started (busno))
+  if (not_started (busno))
     return -RTEMS_NOT_OWNER_OF_RESOURCE;
 
-  sc = bush->ops->write_bytes (bush, bytes, nbytes);
+  sc = bush->ops->write_bytes (bush, (unsigned char *)bytes, nbytes);
   if (sc < 0)
     rtems_libi2c_send_stop (minor);
   return sc;
 }
 
-static int
-do_s_rw (uint32_t minor, unsigned char *bytes, int nbytes, int rw)
+int
+rtems_libi2c_ioctl (rtems_device_minor_number minor,
+		    int cmd,
+		    ...)
 {
-  rtems_status_code sc;
+  va_list            ap;
+  int sc = 0;
+  void *args;
+  bool is_started = false;
+  DECL_CHECKED_BH (busno, bush, minor, -)
+
+  va_start(ap, cmd);
+  args = va_arg(ap, void *);
+
+  switch(cmd) {
+    /*
+     * add ioctls defined for this level here:
+     */
+
+  case RTEMS_LIBI2C_IOCTL_GET_DRV_T:
+    /*
+     * query driver table entry
+     */
+    *(rtems_libi2c_drv_t **)args = (drvs[MINOR2DRV(minor)-1].drv);
+    break;
+
+  case RTEMS_LIBI2C_IOCTL_START_TFM_READ_WRITE:
+    if (not_started (busno))
+      return -RTEMS_NOT_OWNER_OF_RESOURCE;
+
+    /*
+     * address device, then set transfer mode and perform read_write transfer
+     */
+    /*
+     * perform start/address
+     */
+    if (sc == 0) {
+      sc = rtems_libi2c_send_start (minor);
+      is_started = (sc == 0);
+    }
+    /*
+     * set tfr mode
+     */
+    if (sc == 0) {
+      sc = bush->ops->ioctl
+	(bush,
+	 RTEMS_LIBI2C_IOCTL_SET_TFRMODE,
+	 &((rtems_libi2c_tfm_read_write_t *)args)->tfr_mode);
+    }
+    /*
+     * perform read_write
+     */
+    if (sc == 0) {
+      sc = bush->ops->ioctl
+	(bush,
+	 RTEMS_LIBI2C_IOCTL_READ_WRITE,
+	 &((rtems_libi2c_tfm_read_write_t *)args)->rd_wr);
+    }
+    if ((sc < 0) && (is_started)) {
+      rtems_libi2c_send_stop (minor);
+    }
+    break;
+  default:
+    sc = bush->ops->ioctl (bush, cmd, args);
+    break;
+  }
+    return sc;
+}
+
+static int
+do_s_rw (rtems_device_minor_number minor,
+	 unsigned char *bytes,
+	 int nbytes,
+	 int rw)
+{
+  rtems_status_code   sc;
   rtems_libi2c_bus_t *bush;
+  int                 status;
 
   if ((sc = rtems_libi2c_send_start (minor)))
     return -sc;
@@ -538,45 +663,47 @@ do_s_rw (uint32_t minor, unsigned char *bytes, int nbytes, int rw)
   }
 
   if (rw)
-    sc = bush->ops->read_bytes (bush, bytes, nbytes);
+    status = bush->ops->read_bytes (bush, bytes, nbytes);
   else
-    sc = bush->ops->write_bytes (bush, bytes, nbytes);
+    status = bush->ops->write_bytes (bush, bytes, nbytes);
 
-  if (sc < 0) {
+  if (status < 0) {
     rtems_libi2c_send_stop (minor);
   }
-  return sc;
+  return status;
 }
 
 int
-rtems_libi2c_start_read_bytes (uint32_t minor, unsigned char *bytes,
+rtems_libi2c_start_read_bytes (rtems_device_minor_number minor,
+			       unsigned char *bytes,
                                int nbytes)
 {
   return do_s_rw (minor, bytes, nbytes, 1);
 }
 
 int
-rtems_libi2c_start_write_bytes (uint32_t minor, unsigned char *bytes,
+rtems_libi2c_start_write_bytes (rtems_device_minor_number minor,
+				const unsigned char *bytes,
                                 int nbytes)
 {
-  return do_s_rw (minor, bytes, nbytes, 0);
+  return do_s_rw (minor, (unsigned char *)bytes, nbytes, 0);
 }
 
 int
-rtems_libi2c_register_drv (char *name, rtems_libi2c_drv_t * drvtbl,
+rtems_libi2c_register_drv (const char *name, rtems_libi2c_drv_t * drvtbl,
                            unsigned busno, unsigned i2caddr)
 {
   int i;
   rtems_status_code err;
   rtems_device_minor_number minor;
 
-  if (!libmutex) {
-    fprintf (stderr, DRVNM " library not initialized\n");
+  if (libmutex == RTEMS_ID_NONE) {
+    safe_printf ( DRVNM "Library not initialized\n");
     return -RTEMS_NOT_DEFINED;
   }
 
   if (name && strchr (name, '/')) {
-    fprintf (stderr, "Invalid name: '%s' -- must not contain '/'\n", name);
+    safe_printf ( DRVNM "Invalid name: '%s' -- must not contain '/'\n", name);
     return -RTEMS_INVALID_NAME;
   }
 
@@ -585,8 +712,8 @@ rtems_libi2c_register_drv (char *name, rtems_libi2c_drv_t * drvtbl,
     return -RTEMS_INVALID_NUMBER;
   }
 
-  if (drvtbl->size < sizeof (*drvtbl)) {
-    fprintf (stderr, DRVNM " drv-ops size too small -- misconfiguration?\n");
+  if (drvtbl == NULL || drvtbl->size < sizeof (*drvtbl)) {
+    safe_printf ( DRVNM "No driver table or size too small -- misconfiguration?\n");
     return -RTEMS_NOT_CONFIGURED;
   }
 
@@ -616,7 +743,7 @@ rtems_libi2c_register_drv (char *name, rtems_libi2c_drv_t * drvtbl,
 
         /* note that 'umask' is applied to 'mode' */
         if (mknod (str, mode, dev)) {
-          fprintf (stderr,
+          safe_printf( DRVNM
                    "Creating device node failed: %s; you can try to do it manually...\n",
                    strerror (errno));
         }
