@@ -27,10 +27,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)ip_input.c	8.2 (Berkeley) 1/4/94
- * $Id: ip_input.c,v 1.8 2007/03/29 14:31:08 ralf Exp $
+ * $Id$
  *	$ANA: ip_input.c,v 1.5 1996/09/18 14:34:59 wollman Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define	_IP_VHL
 
 #include "opt_ipfw.h"
@@ -44,7 +48,7 @@
 #include <sys/domain.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
-#include <sys/errno.h>
+#include <errno.h>
 #include <sys/time.h>
 #include <sys/kernel.h>
 #include <sys/syslog.h>
@@ -184,7 +188,7 @@ void	ipintr(void);
  * All protocols not implemented in kernel go to raw IP protocol handler.
  */
 void
-ip_init()
+ip_init(void)
 {
 	register struct protosw *pr;
 	register int i;
@@ -224,7 +228,7 @@ ip_init()
 
 }
 
-static struct	sockaddr_in ipaddr = { sizeof(ipaddr), AF_INET };
+static struct	sockaddr_in ipaddr = { sizeof(ipaddr), AF_INET, 0, {0}, {0} };
 static struct	route ipforward_rt;
 
 /*
@@ -619,10 +623,7 @@ NETISR_SET(NETISR_IP, ipintr);
  * is given as fp; otherwise have to make a chain.
  */
 static struct ip *
-ip_reass(ip, fp, where)
-	register struct ipasfrag *ip;
-	register struct ipq *fp;
-	struct   ipq    *where;
+ip_reass(struct ipasfrag *ip, struct ipq *fp, struct ipq *where)
 {
 	register struct mbuf *m = dtom(ip);
 	register struct ipasfrag *q;
@@ -800,8 +801,7 @@ dropfrag:
  * associated datagrams.
  */
 static void
-ip_freef(fp)
-	struct ipq *fp;
+ip_freef(struct ipq *fp)
 {
 	register struct ipasfrag *q, *p;
 
@@ -820,8 +820,7 @@ ip_freef(fp)
  * Like insque, but pointers in middle of structure.
  */
 static void
-ip_enq(p, prev)
-	register struct ipasfrag *p, *prev;
+ip_enq(struct ipasfrag *p, struct ipasfrag *prev)
 {
 
 	p->ipf_prev = prev;
@@ -834,8 +833,7 @@ ip_enq(p, prev)
  * To ip_enq as remque is to insque.
  */
 static void
-ip_deq(p)
-	register struct ipasfrag *p;
+ip_deq(struct ipasfrag *p)
 {
 
 	p->ipf_prev->ipf_next = p->ipf_next;
@@ -848,7 +846,7 @@ ip_deq(p)
  * queue, discard it.
  */
 void
-ip_slowtimo()
+ip_slowtimo(void)
 {
 	register struct ipq *fp;
 	int s = splnet();
@@ -874,7 +872,7 @@ ip_slowtimo()
  * Drain off all datagram fragments.
  */
 void
-ip_drain()
+ip_drain(void)
 {
 	int     i;
 
@@ -895,8 +893,7 @@ ip_drain()
  * 0 if the packet should be processed further.
  */
 static int
-ip_dooptions(m)
-	struct mbuf *m;
+ip_dooptions(struct mbuf *m)
 {
 	register struct ip *ip = mtod(m, struct ip *);
 	register u_char *cp;
@@ -1114,8 +1111,7 @@ bad:
  * return internet address info of interface to be used to get there.
  */
 static struct in_ifaddr *
-ip_rtaddr(dst)
-	 struct in_addr dst;
+ip_rtaddr(struct in_addr dst)
 {
 	struct sockaddr_in *sin;
 
@@ -1142,9 +1138,7 @@ ip_rtaddr(dst)
  * to be picked up later by ip_srcroute if the receiver is interested.
  */
 void
-save_rte(option, dst)
-	u_char *option;
-	struct in_addr dst;
+save_rte(u_char *option, struct in_addr dst)
 {
 	unsigned olen;
 
@@ -1166,7 +1160,7 @@ save_rte(option, dst)
  * The first hop is placed before the options, will be removed later.
  */
 struct mbuf *
-ip_srcroute()
+ip_srcroute(void)
 {
 	register struct in_addr *p, *q;
 	register struct mbuf *m;
@@ -1194,7 +1188,7 @@ ip_srcroute()
 	*(mtod(m, struct in_addr *)) = *p--;
 #ifdef DIAGNOSTIC
 	if (ipprintfs)
-		printf(" hops %lx", ntohl(mtod(m, struct in_addr *)->s_addr));
+		printf(" hops %"PRIx32, ntohl(mtod(m, struct in_addr *)->s_addr));
 #endif
 
 	/*
@@ -1214,7 +1208,7 @@ ip_srcroute()
 	while (p >= ip_srcrt.route) {
 #ifdef DIAGNOSTIC
 		if (ipprintfs)
-			printf(" %lx", ntohl(q->s_addr));
+			printf(" %"PRIx32, ntohl(q->s_addr));
 #endif
 		*q++ = *p--;
 	}
@@ -1224,7 +1218,7 @@ ip_srcroute()
 	*q = ip_srcrt.dst;
 #ifdef DIAGNOSTIC
 	if (ipprintfs)
-		printf(" %lx\n", ntohl(q->s_addr));
+		printf(" %"PRIx32"\n", ntohl(q->s_addr));
 #endif
 	return (m);
 }
@@ -1237,9 +1231,7 @@ ip_srcroute()
  * XXX should be deleted; last arg currently ignored.
  */
 void
-ip_stripoptions(m, mopt)
-	register struct mbuf *m;
-	struct mbuf *mopt;
+ip_stripoptions(struct mbuf *m, struct mbuf *mopt)
 {
 	register int i;
 	struct ip *ip = mtod(m, struct ip *);
@@ -1302,7 +1294,7 @@ ip_forward(struct mbuf *m, int srcrt)
 	dest = 0;
 #ifdef DIAGNOSTIC
 	if (ipprintfs)
-		printf("forward: src %lx dst %lx ttl %x\n",
+		printf("forward: src %"PRIx32" dst %"PRIx32" ttl %x\n",
 			ip->ip_src.s_addr, ip->ip_dst.s_addr, ip->ip_ttl);
 #endif
 
@@ -1422,11 +1414,8 @@ ip_forward(struct mbuf *m, int srcrt)
 }
 
 void
-ip_savecontrol(inp, mp, ip, m)
-	register struct inpcb *inp;
-	register struct mbuf **mp;
-	register struct ip *ip;
-	register struct mbuf *m;
+ip_savecontrol(struct inpcb *inp, struct mbuf **mp, struct ip *ip,
+	struct mbuf *m)
 {
 	if (inp->inp_socket->so_options & SO_TIMESTAMP) {
 		struct timeval tv;
