@@ -1,5 +1,12 @@
 /*
- *  $Id: cancel.c,v 1.10 2004/04/16 10:01:03 ralf Exp $
+ *  COPYRIGHT (c) 1989-2008.
+ *  On-Line Applications Research Corporation (OAR).
+ *
+ *  The license and distribution terms for this file may be
+ *  found in the file LICENSE in this distribution or at
+ *  http://www.rtems.com/license/LICENSE.
+ *
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -18,8 +25,7 @@
 #include <rtems/posix/pthread.h>
 #include <rtems/posix/threadsup.h>
 
-/*PAGE
- *
+/*
  *  18.2.1 Canceling Execution of a Thread, P1003.1c/Draft 10, p. 181
  */
 
@@ -27,9 +33,9 @@ int pthread_cancel(
   pthread_t  thread
 )
 {
-  Thread_Control                    *the_thread;
-  POSIX_API_Control                 *thread_support;
-  Objects_Locations                  location;
+  Thread_Control     *the_thread;
+  POSIX_API_Control  *thread_support;
+  Objects_Locations   location;
 
   /*
    *  Don't even think about deleting a resource from an ISR.
@@ -38,25 +44,24 @@ int pthread_cancel(
   if ( _ISR_Is_in_progress() )
     return EPROTO;
 
-  the_thread = _POSIX_Threads_Get( thread, &location );
+  the_thread = _Thread_Get( thread, &location );
   switch ( location ) {
-    case OBJECTS_ERROR:
-      return EINVAL;
-    case OBJECTS_REMOTE:
-      return POSIX_MP_NOT_IMPLEMENTED();
+
     case OBJECTS_LOCAL:
       thread_support = the_thread->API_Extensions[ THREAD_API_POSIX ];
 
       thread_support->cancelation_requested = 1;
 
-      if ( thread_support->cancelability_state == PTHREAD_CANCEL_ENABLE &&
-           thread_support->cancelability_type == PTHREAD_CANCEL_ASYNCHRONOUS ) {
-        _POSIX_Threads_cancel_run( the_thread );
-      }
-
-      _Thread_Enable_dispatch();
+      /* This enables dispatch implicitly */
+      _POSIX_Thread_Evaluate_cancellation_and_enable_dispatch( the_thread );
       return 0;
+
+#if defined(RTEMS_MULTIPROCESSING)
+    case OBJECTS_REMOTE:
+#endif
+    case OBJECTS_ERROR:
+      break;
   }
 
-  return POSIX_BOTTOM_REACHED();
+  return EINVAL;
 }
