@@ -1,14 +1,14 @@
 /*
  *  POSIX RWLock Manager -- Obtain a Write Lock on a RWLock Instance
  *
- *  COPYRIGHT (c) 1989-2006.
+ *  COPYRIGHT (c) 1989-2007.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: prwlockwrlock.c,v 1.1.2.1 2007/11/06 19:54:22 humph Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -40,23 +40,19 @@ int pthread_rwlock_wrlock(
 {
   POSIX_RWLock_Control  *the_rwlock;
   Objects_Locations      location;
-  
+
   if ( !rwlock )
     return EINVAL;
 
   the_rwlock = _POSIX_RWLock_Get( rwlock, &location );
   switch ( location ) {
 
-    case OBJECTS_REMOTE:
-    case OBJECTS_ERROR:
-      return EINVAL;
-
     case OBJECTS_LOCAL:
 
       _CORE_RWLock_Obtain_for_writing(
 	&the_rwlock->RWLock,
 	*rwlock,
-	TRUE,          /* do not timeout -- wait forever */
+	true,          /* do not timeout -- wait forever */
 	0,
 	NULL
       );
@@ -65,7 +61,13 @@ int pthread_rwlock_wrlock(
       return _POSIX_RWLock_Translate_core_RWLock_return_code(
         (CORE_RWLock_Status) _Thread_Executing->Wait.return_code
       );
+
+#if defined(RTEMS_MULTIPROCESSING)
+    case OBJECTS_REMOTE:
+#endif
+    case OBJECTS_ERROR:
+      break;
   }
 
-  return POSIX_BOTTOM_REACHED();
+  return EINVAL;
 }
