@@ -12,7 +12,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: corerwlock.c,v 1.2 2007/06/20 19:21:01 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -24,9 +24,6 @@
 #include <rtems/score/states.h>
 #include <rtems/score/thread.h>
 #include <rtems/score/threadq.h>
-#if defined(RTEMS_MULTIPROCESSING)
-#include <rtems/score/mpci.h>
-#endif
 
 /*PAGE
  *
