@@ -7,14 +7,14 @@
  *
  *  Output parameters:  NONE
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2009.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: task1.c,v 1.8 2007/05/11 19:44:17 joel Exp $
+ *  $Id$
  */
 
 #include "system.h"
@@ -28,7 +28,7 @@ void Task_1()
    */
 
   puts( "TA1 - rtems_task_wake_after - sleep 1 second" );
-  status = rtems_task_wake_after( 1*TICKS_PER_SECOND );
+  status = rtems_task_wake_after( 1*rtems_clock_get_ticks_per_second() );
   directive_failed( status, "rtems_task_wake_after" );
 
   status = chg_pri( TA3_ID , 2 );
@@ -49,7 +49,7 @@ void Task_1()
    */
 
   puts( "TA1 - rtems_task_wake_after - sleep for 5 seconds" );
-  status = rtems_task_wake_after( 5*TICKS_PER_SECOND );
+  status = rtems_task_wake_after( 5*rtems_clock_get_ticks_per_second() );
   directive_failed( status, "rtems_task_wake_after" );
 
   puts( "*** END OF ITRON TASK TEST 3 ***" );
