@@ -3,14 +3,14 @@
  *  This set of three tasks do some simple task switching for about
  *  15 seconds and then call a routine to "blow the stack".
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2009.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: task1.c,v 1.8 2003/09/04 18:53:27 joel Exp $
+ *  $Id$
  */
 
 #include "system.h"
@@ -27,17 +27,18 @@ rtems_task Task_1_through_3(
   directive_failed( status, "rtems_task_ident" );
 
   while( FOREVER ) {
-    status = rtems_clock_get( RTEMS_CLOCK_GET_TOD, &time );
-    directive_failed( status, "rtems_clock_get" );
+    status = rtems_clock_get_tod( &time );
+    directive_failed( status, "rtems_clock_get_tod" );
 
     if ( time.second >= 15 && tid == Task_id[ 1 ] ) {
        blow_stack();
     }
 
     put_name( Task_name[ task_number( tid ) ], FALSE );
-    print_time( " - rtems_clock_get - ", &time, "\n" );
+    print_time( " - rtems_clock_get_tod - ", &time, "\n" );
 
-    status = rtems_task_wake_after( task_number( tid ) * 5 * TICKS_PER_SECOND );
+    status = rtems_task_wake_after(
+      task_number( tid ) * 5 * rtems_clock_get_ticks_per_second() );
     directive_failed( status, "rtems_task_wake_after" );
   }
 }
