@@ -7,14 +7,14 @@
  *
  *  Output parameters:  NONE
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2009.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: task2.c,v 1.7 2003/09/04 18:53:38 joel Exp $
+ *  $Id$
  */
 
 #include "system.h"
@@ -26,18 +26,18 @@ void *Task_2(
   int  status;
 
   Task_id = pthread_self();
-  printf( "Task_2: ID is 0x%08x\n", Task_id );
+  printf( "Task_2: ID is 0x%08" PRIxpthread_t "\n", Task_id );
 
   status = pthread_mutex_lock( &Mutex_id );
-  assert( !status );
+  rtems_test_assert(  !status );
 
   puts( "Task_2: pthread_cond_wait" );
   status = pthread_cond_wait( &Cond1_id, &Mutex_id );
-  assert( !status );
+  rtems_test_assert(  !status );
 
   puts( "Task_2: back from pthread_cond_wait release mutex" );
   status = pthread_mutex_unlock( &Mutex_id );
-  assert( !status );
+  rtems_test_assert(  !status );
 
   puts( "Task_2: task exit" );
   pthread_exit( NULL );
