@@ -2,14 +2,14 @@
  *  This test file is used to verify that the header files associated with
  *  invoking this function are correct.
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2009.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: pthread31.c,v 1.8 2004/04/16 09:23:26 ralf Exp $
+ *  $Id$
  */
 
 #include <pthread.h>
@@ -18,6 +18,9 @@
 #error "rtems is supposed to have pthread_testcancel"
 #endif
 
+void test( void );
+void init_routine( void );
+
 void init_routine( void )
 {
 }
